<?php
// $Id: duration.api.inc,v 1.2 2008/03/22 01:18:58 jpetso Exp $

/**
 * @file
 * A form element for entering time durations.
 * This file contains functions for transformations of / evaluations with
 * duration arrays.
 *
 * Copyright 2007, 2008 by Jakob Petsovits <jpetso@gmx.at>
 * Distributed under the GNU General Public Licence version 2 or higher,
 * as published by the FSF on http://www.gnu.org/copyleft/gpl.html
 */


/**
 * Format a duration array as string.
 */
function duration_format($duration) {
  $metrics_info = _duration_metrics_info('descending');
  $strings = array();

  foreach ($metrics_info as $metric => $info) {
    if (!isset($duration[$metric]) || $duration[$metric] <= 0) {
      continue;
    }
    $strings[] = format_plural(
      $duration[$metric], $info['number_singular'], $info['number_plural']
    );
  }
  return implode(', ', $strings);
}

/**
 * Add a timespan to a given point in time.
 *
 * @param $duration
 *   The duration array that will be added to the timestamp. The array should
 *   look like one that is returned by the date combo and can hold any elements,
 *   but only the 'hours', 'minutes', 'seconds', 'mon', 'mday' and 'year'
 *   elements will be taken into account for this function.
 * @param $timestamp
 *   The time to which the duration is added. Defaults to the current time.
 */
function duration_add_to_time($duration, $timestamp = NULL) {
  if (!isset($timestamp)) {
    $timestamp = time();
  }
  $date = getdate($timestamp);

  foreach ($duration as $metric => $value) {
    $date[$metric] += $value;
  }
  return mktime(
    $date['hours'], $date['minutes'], $date['seconds'],
    $date['mon'], $date['mday'], $date['year']
  );
}

/**
 * Subtract a timespan to a given point in time.
 *
 * @param $duration
 *   The duration array that will be subtracted from the timestamp. The array
 *   should look like one that is returned by the date combo and can hold any
 *   elements, but only the 'hours', 'minutes', 'seconds', 'mon', 'mday' and
 *   'year' elements will be taken into account for this function.
 * @param $timestamp
 *   The time from which the duration is subtracted.
 *   Defaults to the current time.
 */
function duration_subtract_from_time($duration, $timestamp = NULL) {
  if (!isset($timestamp)) {
    $timestamp = time();
  }
  $date = getdate($timestamp);

  foreach ($duration as $metric => $value) {
    $date[$metric] -= $value;
  }
  return mktime(
    $date['hours'], $date['minutes'], $date['seconds'],
    $date['mon'], $date['mday'], $date['year']
  );
}

/**
 * Set all metrics smaller than @p $smallest_metric to 0 and add their values
 * to the @p $smallest_metric value using the appropriate conversion factor.
 *
 * Note that the 'month' and 'year' metrics are likely to cause inaccurate
 * results because months and years have differences in length depending on
 * which month or year this applies to. In this function, months are calculated
 * by using a conversion factor of 30 (days), and 365 is used for years.
 */
function duration_limit_granularity($duration, $smallest_metric) {
  $duration = _duration_fill_missing_elements($duration);
  if ($smallest_metric == 'milliseconds') {
    return $duration;
  }
  $metrics_info = _duration_metrics_info('ascending');
  array_shift($metrics_info); // get rid of the 'milliseconds' info, not needed here

  foreach ($metrics_info as $metric => $info) {
    $duration[$metric] += $duration[$info['smaller_metric']] / $info['unit_count'];
    if ($metric == $smallest_metric) {
      break;
    }
  }
  return $duration;
}

/**
 * Make sure that no part of the duration array contains a higher value
 * than the highest value that is normally used for this metric. Also,
 * the decimal place of a larger unit is pushed to the respective smaller unit.
 * Example: a duration of 65.5 seconds would be transformed into
 * 1 minute, 5 seconds and 500 milliseconds.
 *
 * Note that the 'month' and 'year' metrics are likely to cause inaccurate
 * results because months and years have differences in length depending on
 * which month or year this applies to. In this function, months are calculated
 * by using a conversion factor of 30 (days), and 365 is used for years.
 */
function duration_normalize($duration) {
  $duration = _duration_fill_missing_elements($duration);
  $metrics_info = _duration_metrics_info('descending');

  // Transfer decimal place values from the larger units to the smaller ones.
  foreach ($metrics_info as $metric => $info) {
    if (!isset($info['smaller_metric'])) {
      break;
    }
    $decimal_place = $duration[$metric] - floor($duration[$metric]);
    $duration[$info['smaller_metric']] += $decimal_place * $info['unit_count'];
    $duration[$metric] = (int) ($duration[$metric] - $decimal_place);
  }
  // That's that, now let's do the overflow thing in the reverse direction.
  $metrics_info = array_reverse($metrics_info);

  // Add overflowing units from the smaller units to the larger ones.
  foreach ($metrics_info as $metric => $info) {
    if (!isset($info['larger_metric'])) {
      break;
    }
    $larger_metric_info = $metrics_info[$info['larger_metric']];

    if ($duration[$metric] > $larger_metric['unit_count']) {
      $duration[$info['larger_metric']] += $duration[$metric] / $larger_metric_info['unit_count'];
      $duration[$metric] %= $larger_metric_info['unit_count'];
    }
  }
  return $duration;
}

/**
 * Get the largest metric that is assigned a positive value.
 * If no part of the duration array holds a positive value then the largest
 * known metric overall is returned, which is 'year'.
 */
function duration_get_largest_metric(&$duration) {
  $duration = _duration_fill_missing_elements($duration);

  foreach (_duration_metrics('descending') as $metric) {
    if ($duration[$metric] > 0) {
      return $metric;
    }
  }
  return 'year';
}

/**
 * Get the smallest metric that is assigned a positive value.
 * If no part of the duration array holds a positive value then the smallest
 * known metric overall is returned, which is 'milliseconds'.
 */
function duration_get_smallest_metric(&$duration) {
  $duration = _duration_fill_missing_elements($duration);

  foreach (_duration_metrics('ascending') as $metric) {
    if ($duration[$metric] > 0) {
      return $metric;
    }
  }
  return 'milliseconds';
}


/**
 * Fill up an incomplete durations array so that all known elements are set.
 * Missing elements will be filled with 0.
 */
function _duration_fill_missing_elements($duration) {
  foreach (_duration_metrics() as $metric) {
    if (!isset($duration[$metric])) {
      $duration[$metric] = 0;
    }
  }
  return $duration;
}


function _duration_metrics($sort = 'ascending') {
  return array_keys(_duration_metrics_info($sort));
}

function _duration_metrics_specific_info($which, $sort = 'ascending') {
  $metrics = array();
  foreach (_duration_metrics_info($sort) as $metric => $info) {
    $metrics[$metric] = $info[$which];
  }
  return $metrics;
}

function _duration_metrics_info($sort = 'ascending') {
  $metrics = array(
    'milliseconds' => array(
      'plain_plural' => t('milliseconds'),
      'number_singular' => t('1 millisecond'),
      'number_plural' => t('@count milliseconds'),
      'larger_metric' => 'seconds',
    ),
    'seconds' => array(
      'plain_plural' => t('seconds'),
      'number_singular' => t('1 second'),
      'number_plural' => t('@count seconds'),
      'unit_count' => 1000.0,
      'smaller_metric' => 'milliseconds',
      'larger_metric' => 'minutes',
    ),
    'minutes' => array(
      'plain_plural' => t('minutes'),
      'number_singular' => t('1 minute'),
      'number_plural' => t('@count minutes'),
      'unit_count' => 60.0,
      'smaller_metric' => 'seconds',
      'larger_metric' => 'hours',
    ),
    'hours' => array(
      'plain_plural' => t('hours'),
      'number_singular' => t('1 hour'),
      'number_plural' => t('@count hours'),
      'unit_count' => 60.0,
      'smaller_metric' => 'minutes',
      'larger_metric' => 'mday',
    ),
    'mday' => array(
      'plain_plural' => t('days'),
      'number_singular' => t('1 day'),
      'number_plural' => t('@count days'),
      'unit_count' => 24.0,
      'smaller_metric' => 'hours',
      'larger_metric' => 'mon',
    ),
    'mon' => array(
      'plain_plural' => t('months'),
      'number_singular' => t('1 month'),
      'number_plural' => t('@count months'),
      'unit_count' => 30.0,
      'smaller_metric' => 'mday',
      'larger_metric' => 'year',
    ),
    'year' => array(
      'plain_plural' => t('years'),
      'number_singular' => t('1 year'),
      'number_plural' => t('@count years'),
      'unit_count' => 365.0,
      'smaller_metric' => 'mon',
    ),
  );
  if ($sort = 'descending') {
    return array_reverse($metrics, TRUE);
  }
  return $metrics;
}
